from piexin.phpipam.address import Address
from piexin.entrypoint import create_argparser


class TestValidateFqdn:

    good_names = [
        "host1.localdomain.local",
        "host2.localdomain.local",
        "host1.localdomain.local.world",
        "host1.other.localdomain.local"
    ]

    bad_names = [
        "host1..localdomain.local",
        "host2.local",
        "host1..localdomain.local.world",
        "host1"
    ]

    host_json_skeleton = {
        "id": 1,
        "ip": 1,
        "subnetId": 1
    }

    def test_good_names(self):

        for name in TestValidateFqdn.good_names:

            json = TestValidateFqdn.host_json_skeleton
            json["hostname"] = name

            addr = Address(json)

            assert name and addr.validate_fqdn()

    def test_bad_names(self):

        for name in TestValidateFqdn.bad_names:

            json = TestValidateFqdn.host_json_skeleton
            json["hostname"] = name

            addr = Address(json)

            assert name and not addr.validate_fqdn()



import argparse
import sys
from piexin.phpipam.phpipam import Phpipam
from piexin.phpipam.exeption import PhpipamException
from piexin.inventorywriter.inventorywriter import Inventorywriter
import piexin

def create_argparser(args):
    """
    parse args
    """
    argp = argparse.ArgumentParser(description=__doc__)
    argp.add_argument(
        '-u', '--url',
        metavar='url', default='http://localhost', type=str,
        help='url of phpipam server api, defaults to http://localhost'
    )
    argp.add_argument(
        '-t', '--token',
        metavar='token', default='', type=str,
        help='v4 api token'
    )
    argp.add_argument(
        '-a', '--app',
        metavar='app', default='', type=str,
        help='phpipam api appname'
    )
    argp.add_argument(
        '-c', '--cert',
        metavar='certificate', default='', type=str,
        help='ssl cert from server if server is not trusted'
    )
    argp.add_argument(
        '-o', '--output',
        metavar='output', default='', type=str,
        help='file to print inventory to'
    )
    argp.add_argument(
        '-V', '--version',
        default=False, action='store_true',
        help='print version'
    )

    argp.add_argument('-v', '--verbose', action='count', default=0)

    return argp.parse_args(args)


def main():
    args = create_argparser(sys.argv[1:])

    if args.version:
        print(piexin.__version__)
        sys.exit()

    try:
        ipam = Phpipam(args)
        inventorywriter = Inventorywriter(ipam)

        ipam.dump()
        inventorywriter.write(args.output)

    except PhpipamException as e:
        print(e.message, file=sys.stderr)
        sys.exit(1)

    except Exception as e:
        print(e, file=sys.stderr)
        sys.exit(1)


if __name__ == '__main__':
    main()



import requests
import os
import re
from .exeption import PhpipamException
from .section import Section
from .subnet import Subnet
from .address import Address
from .vlan import Vlan


class Phpipam:

    key_env_token = 'PHPIPAM_TOKEN'

    def __init__(self, args):

        self.url = '{0}/api/{1}'.format(args.url, args.app)

        self.headers = {}
        if args.token == '' and Phpipam.key_env_token in os.environ:
            self.headers['token'] = os.environ[Phpipam.key_env_token]
        else:
            self.headers['token'] = args.token

        self.verify = True
        if args.cert != '':
            self.verify = args.cert

        self.vlans = []
        self.addresses = []
        self.subnets = []
        self.sections = []

    def __request(self, req):

        url = self.url + req
        headers = self.headers
        verify = self.verify
        ret = requests.get(url, headers=headers, verify=verify).json()

        if ret['code'] != 200:
            raise PhpipamException(ret['message'])

        if 'data' not in ret:
            return []  # empty response
        return ret['data']

    def dump(self):

        for json in self.__request('/sections'):

            section = Section(json)
            self.sections.append(section)

            subnets_json = self.__request(
                '/sections/{0}/subnets'.format(section.id))
            for subnet_json in subnets_json:
                subnet = Subnet(subnet_json)
                section.subnets.append(subnet)
                self.subnets.append(subnet)

                address_json = self.__request(
                    '/subnets/{0}/addresses/'.format(subnet.id))
                for address_json in address_json:
                    address = Address(address_json)

                    if not address.validate_fqdn():
                        continue

                    subnet.addresses.append(address)
                    self.addresses.append(address)

                if subnet.vlan_id != '0':
                    vlan_json = self.__request(
                                '/vlan/{0}'.format(subnet.vlan_id))
                    if len(vlan_json) > 0:
                        subnet.set_vlan(Vlan(vlan_json))

    def get_addresses(self):
        return self.addresses

    def get_subnets(self):
        return self.subnets

    def get_vlans(self):
        return self.vlans




import re


class Address:

    hostname_regex = re.compile('^[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?$', re.IGNORECASE)

    def __init__(self, host_json):

        self.id = host_json["id"]
        self.ip = host_json["ip"]
        self.hostname = host_json["hostname"]
        self.subnet_id = host_json["subnetId"]

        self.ansible_groups = []
        if 'custom_ansible_groups' in host_json:
            self.ansible_groups = \
                self.parse_ansible_groups(host_json["custom_ansible_groups"])

    def parse_ansible_groups(self, string):

        ret_ary = []
        if not isinstance(string, str):
            return ret_ary

        names_ary = string.split(',')

        re_string = '^[^0-9_][a-zA-Z]+[a-zA-Z0-9_]*$'
        pattern = re.compile(re_string)

        for name in names_ary:
            if pattern.match(name):
                ret_ary.append(name.replace(' ', ''))

        return ret_ary

    def validate_fqdn(self):

        if len(self.hostname) > 253:
            return False

        hostname_parts = self.hostname.split('.')

        if len(hostname_parts) < 3:
            return False

        return all(Address.hostname_regex.match(part) for part in hostname_parts)




class Vlan:

    def __init__(self, vlan_json):

        self.id = vlan_json["vlanId"]
        self.vlan_id = vlan_json["number"]

        self.name = ''
        if 'name' in vlan_json:
            self.name = vlan_json["name"]



import pathlib
import os

import piexin


class TestCommand:

    inventory_file_content = '[piexin_all]\n' \
                             'host1.localdomain.local ansible_host=10.0.0.1\n' \
                             'host2.localdomain.local ansible_host=10.0.0.2\n' \
                             'host3.localdomain.local ansible_host=10.0.1.1\n' \
                             'host4.localdomain.local ansible_host=10.0.1.2\n' \
                             'host5.localdomain.local ansible_host=172.16.0.1\n' \
                             'host6.localdomain.local ansible_host=172.16.0.2\n' \
                             'host7.localdomain.local ansible_host=172.16.1.1\n' \
                             'host8.localdomain.local ansible_host=172.16.1.2\n' \
                             '\n' \
                             '[dmz]\n' \
                             'host1.localdomain.local\n' \
                             'host5.localdomain.local\n' \
                             '\n' \
                             '[group1]\n' \
                             'host1.localdomain.local\n' \
                             '\n' \
                             '[group2]\n' \
                             'host2.localdomain.local\n' \
                             'host5.localdomain.local\n' \
                             '\n' \
                             '[web1]\n' \
                             'host3.localdomain.local\n' \
                             'host4.localdomain.local\n' \
                             '\n' \
                             '[piexin_subnet_10_0_0_0]\n' \
                             'host1.localdomain.local\n' \
                             'host2.localdomain.local\n' \
                             '\n' \
                             '[piexin_subnet_10_0_0_0:vars]\n' \
                             'piexin_subnet = 10.0.0.0\n' \
                             'piexin_subnet_mask = 24\n' \
                             'piexin_vlan_id = 3\n' \
                             'piexin_vlan_name = 10_FIRST\n' \
                             '\n' \
                             '[piexin_subnet_10_0_1_0]\n' \
                             'host3.localdomain.local\n' \
                             'host4.localdomain.local\n' \
                             '\n' \
                             '[piexin_subnet_10_0_1_0:vars]\n' \
                             'piexin_subnet = 10.0.1.0\n' \
                             'piexin_subnet_mask = 24\n' \
                             'piexin_vlan_id = 4\n' \
                             'piexin_vlan_name = 11_SCND\n' \
                             '\n' \
                             '[piexin_subnet_172_16_0_0]\n' \
                             'host5.localdomain.local\n' \
                             'host6.localdomain.local\n' \
                             '\n' \
                             '[piexin_subnet_172_16_0_0:vars]\n' \
                             'piexin_subnet = 172.16.0.0\n' \
                             'piexin_subnet_mask = 24\n' \
                             'piexin_vlan_id = 5\n' \
                             'piexin_vlan_name = 12_THRD\n' \
                             '\n' \
                             '[piexin_subnet_172_16_1_0]\n' \
                             'host7.localdomain.local\n' \
                             'host8.localdomain.local\n' \
                             '\n' \
                             '[piexin_subnet_172_16_1_0:vars]\n' \
                             'piexin_subnet = 172.16.1.0\n' \
                             'piexin_subnet_mask = 24\n' \
                             'piexin_vlan_id = 6\n' \
                             'piexin_vlan_name = 13_4TH\n' \
                             '\n'

    def test_help(self, script_runner):
        version = piexin.__version__

        ret = script_runner.run('piexin', '--version')

        assert ret.success
        assert ret.stdout == '{0}\n'.format(version)
        assert ret.stderr == ''

    def test_no_application_given(self, script_runner):
        ret = script_runner.run('piexin')

        assert not ret.success
        assert ret.stdout == ''
        assert ret.stderr == 'phpipam error: Invalid application id\n'

    def test_no_token_given(self, script_runner):
        ret = script_runner.run('piexin', '-a', 'development')

        assert not ret.success
        assert ret.stdout == ''
        assert ret.stderr == 'phpipam error: Unauthorized\n'

    def test_wrong_token_given(self, script_runner):
        ret = script_runner.run('piexin', '-a', 'development', '-t', 'invalid')

        assert not ret.success
        assert ret.stdout == ''
        assert ret.stderr == 'phpipam error: Unauthorized\n'

    def test_no_cert_given(self, script_runner):
        ret = script_runner.run(
            'piexin', '-a', 'development',
            '-t', 'developcode',
            '-u', 'https://localhost')

        assert not ret.success
        assert ret.stdout == ''
        assert ret.stderr != ''

    def test_with_server_cert(self, script_runner):

        path = pathlib.Path(__file__).parent.absolute().as_posix()

        ret = script_runner.run(
            'piexin',
            '-a', 'development',
            '-t', 'developcode',
            '-u', 'https://localhost',
            '-c', path + '/integration/nginx.crt'
        )

        assert ret.stderr == ''
        assert ret.stdout != ''
        assert ret.success

    def test_successful_params(self, script_runner):
        ret = script_runner.run('piexin', '-a', 'development', '-t', 'developcode')

        assert ret.success
        assert ret.stdout != ''
        assert ret.stderr == ''

        assert ret.stdout == TestCommand.inventory_file_content

    def test_successfule_environ(self, script_runner):

        env = {'PHPIPAM_TOKEN': 'developcode'}

        ret = script_runner.run('piexin', '-a', 'development', env=env)

        assert ret.success
        assert ret.stdout != ''
        assert ret.stderr == ''

        assert ret.stdout == TestCommand.inventory_file_content

    def test_file_output(self, script_runner):

        testfile = '/tmp/testfile.ini'

        if os.path.isfile(testfile):
            os.remove(testfile)

        ret = script_runner.run('piexin',
                                '-a', 'development',
                                '-t', 'developcode',
                                '-o', testfile)

        assert ret.success
        assert ret.stdout == ''
        assert ret.stderr == ''

        with open(testfile, 'r') as f:
            content = f.read()

        assert content == TestCommand.inventory_file_content

        os.remove(testfile)



import os


class Inventorywriter:

    def __init__(self, src):

        self.src = src

    def write(self, filename):

        new_content = self.get_content_from_source()

        if filename is None or not isinstance(filename, str) or filename == '':
            print(new_content, end='')
            return True

        if os.path.isfile(filename):
            with open(filename) as f:
                existing_content = f.read()
            if existing_content == new_content:
                return True

        with open(filename, 'w') as f:
            f.write(new_content)

    def get_content_from_source(self):

        lines_array_of_arrays = [
            self.get_hosts(),
            self.get_subnets()
        ]

        lines = []
        for ary in lines_array_of_arrays:
            for line in ary:
                lines.append(line)

        return ''.join(lines)

    def get_hosts(self):

        lines = ['[piexin_all]\n']
        groups = {}

        for address in self.src.get_addresses():

            for group in address.ansible_groups:
                if group not in groups:
                    groups[group] = []
                groups[group].append(address.hostname)

            attributes = {
                'ansible_host': address.ip
            }

            attlist = []
            for key, value in attributes.items():
                attlist.append('{0}={1}'.format(key, value))

            lines.append('{0} {1}\n'.format(address.hostname, ''.join(attlist)))

        lines.append('\n')

        for groupname, members in groups.items():
            lines.append('[{0}]\n'.format(groupname))

            for member in members:
                lines.append('{0}\n'.format(member))

            lines.append('\n')

        return lines

    def get_subnets(self):

        lines = []

        for subnet in self.src.get_subnets():

            if len(subnet.addresses) == 0:
                continue

            net_address = subnet.subnet.replace('.', '_')
            group_name = 'piexin_subnet_{0}'.format(net_address)

            # place hosts in group
            lines.append('[{0}]\n'.format(group_name))
            for address in subnet.addresses:
                lines.append('{0}\n'.format(address.hostname))
            lines.append('\n')

            # group vars
            group_vars = [
                '[{0}:vars]\n'.format(group_name),
                'piexin_subnet = {0}\n'.format(subnet.subnet),
                'piexin_subnet_mask = {0}\n'.format(subnet.mask),
            ]
            if subnet.vlan_id != '0':
                group_vars.extend([
                    'piexin_vlan_id = {0}\n'.format(subnet.vlan_id),
                    'piexin_vlan_name = {0}\n'.format(subnet.vlan.name)
                ])
            lines.extend(group_vars)
            lines.append('\n')

        return lines







class PhpipamException(Exception):

    def __init__(self, message):

        prefix = 'phpipam error: '

        if isinstance(message, str):
            self.message = prefix + message
        else:
            self.message = prefix + 'unknown error'



from piexin.phpipam.address import Address


class TestAddress:

    def __json(self):

        return {
            "id": "bla",
            "ip": "bla",
            "hostname": "bla",
            "subnetId": "bla"
        }

    def test_parse_ansible_groups(self):

        address = Address(self.__json())

        string = 'web, group1'

        group_ary = address.parse_ansible_groups(string)

        assert 'web' in group_ary
        assert 'group1' in group_ary
        assert len(group_ary) == 2

    def test_parse_ansible_groups_names(self):

        test_ary = [
            'web',
            'group1',
            'group1_2',
            'anotherGroup'
        ]

        address = Address(self.__json())

        string = ''
        for s in test_ary:
            string = string + s + ', '

        group_ary = address.parse_ansible_groups(string)

        for ix in test_ary:
            assert ix in group_ary

        assert len(group_ary) == len(test_ary)

    def test_parse_ansible_groups_negative(self):

        address = Address(self.__json())

        string = 'web, 0group1, _group1'

        group_ary = address.parse_ansible_groups(string)

        assert 'web' in group_ary
        assert '0group1' not in group_ary
        assert '_group1' not in group_ary
        assert len(group_ary) == 1



import piexin
from setuptools import setup
import setuptools

setup(
    name='piexin',
    version=piexin.__version__,
    description='phpipam export to ansible inventory ini file',
    url='http://github.com/aufbaubank/piexin',
    author='Daniel Henneberg',
    author_email='daniel.henneberg@aufbaubank.de',
    license='MIT',
    entry_points={
        'console_scripts': ['piexin=piexin.entrypoint:main'],
    },
    zip_safe=False,
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=['requests']
)



class Subnet:

    def __init__(self, subnet_json):

        self.id = subnet_json["id"]
        self.subnet = subnet_json["subnet"]
        self.mask = subnet_json["mask"]
        self.vlan_id = subnet_json["vlanId"]
        self.vlan = None

        self.addresses = []

    def set_vlan(self, vlan):

        self.vlan = vlan
